@@ -26,7 +26,8 @@ Execution <- list(execId=        "",
                   evMultiplier=  0,
                   modelCode=     "",
                   lastLiquidity= 0L,
-                  pendingPriceRevision= FALSE)
+                  pendingPriceRevision= FALSE,
+                  submitter=     "")
 
 ExecutionFilter <- list(clientId= 0L,
                         acctCode= "",
@@ -153,6 +154,19 @@ OrderState  <- list(status=               "",
                     minCommission=        NA_real_,
                     maxCommission=        NA_real_,
                     commissionCurrency=   "",
+                    marginCurrency=       "",
+                    initMarginBeforeOutsideRTH=     NA_real_,
+                    maintMarginBeforeOutsideRTH=    NA_real_,
+                    equityWithLoanBeforeOutsideRTH= NA_real_,
+                    initMarginChangeOutsideRTH=     NA_real_,
+                    maintMarginChangeOutsideRTH=    NA_real_,
+                    equityWithLoanChangeOutsideRTH= NA_real_,
+                    initMarginAfterOutsideRTH=      NA_real_,
+                    maintMarginAfterOutsideRTH=     NA_real_,
+                    equityWithLoanAfterOutsideRTH=  NA_real_,
+                    suggestedSize=        NA_real_,
+                    rejectReason=         "",
+                    orderAllocations=     list(),
                     warningText=          "",
                     completedTime=        "",
                     completedStatus=      "")
@@ -300,7 +314,8 @@ Order <- list(orderId=                        0L,
               professionalCustomer=           FALSE,
               bondAccruedInterest=            "",
               includeOvernight=               FALSE,
-              manualOrderIndicator=           NA_integer_)
+              manualOrderIndicator=           NA_integer_,
+              submitter=                      "")
 
 OrderCancel <- list(manualOrderCancelTime= "",
                     extOperator=           "",
